//----------------------------------------------------------------------------------------------
// Board Library : Seeed nRF52 Borads 1.1.8
// Board Select  : Seeed nRF52 Borads / Seeed XIAO nRF52840 Sense
// 2024/04/28 @msfujino
//----------------------------------------------------------------------------------------------

#include <bluefruit.h>      // \Arduino15\packages\Seeeduino\hardware\nrf52\1.1.8\libraries\Bluefruit52Lib\src
#define dataNum     8       // send data number : millis(), micros() (2 data 8 byte)

union unionData {                   //Union for bit convertion
  uint32_t  dataBuff32[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];  
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

// Custum Service and Characteristic
// generated Linux command uuidgen, Example "0dd7eb5a-a8b9-4f45-bcd7-94c674c3b25f" 
#define myUUID(val) (const uint8_t[]) { \
    0x5F, 0xB2, 0xC3, 0x74, 0xC6, 0x94, 0xD7, 0xBC, \
    0x45, 0x4F, (uint8_t)(val & 0xff), (uint8_t)(val >> 8), 0x5A, 0xEB, 0xD7, 0x0D }
BLEService        myService           (myUUID(0x0000));
BLECharacteristic myCharacteristic    (myUUID(0x0030));

// ************************************************************************************************
void setup() {
  // initialization of serial port
  Serial.begin(115200);
//  while (!Serial);
  delay(2000);

  // I/O pins settings
  pinMode(LED_RED, OUTPUT);         // LOW:LED ON, HIGH:LED OFF
  pinMode(LED_GREEN, OUTPUT);
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);

  // Initialization of Bruefruit class
  Bluefruit.configPrphBandwidth(BANDWIDTH_MAX);
  Bluefruit.configUuid128Count(15);

  Bluefruit.begin();
  Bluefruit.setTxPower(4);
  Bluefruit.setConnLedInterval(50);   // Blue LED
  Bluefruit.Periph.setConnectCallback(connect_callback);
  Bluefruit.Periph.setDisconnectCallback(disconnect_callback);

  // Custom Service Settings
  myService.begin();

  myCharacteristic.setProperties(CHR_PROPS_NOTIFY);
  myCharacteristic.setFixedLen(dataNum);
  myCharacteristic.begin();

  // Advertisement Settings
  Bluefruit.Advertising.addFlags(BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE);
  Bluefruit.Advertising.addTxPower();
  Bluefruit.Advertising.addService(myService);
  Bluefruit.ScanResponse.addName();
  
  Bluefruit.Advertising.restartOnDisconnect(true);
  Bluefruit.Advertising.setIntervalMS(20, 153);     // fast mode 20mS, slow mode 153mS
  Bluefruit.Advertising.setFastTimeout(30);         // fast mode 30 sec
  Bluefruit.Advertising.start(0);                   // 0 = Don't stop advertising after n seconds

  Serial.println("End of initialization");
}

void loop() {
  // connecting to Central
  Serial.println("Connecting to Central ................");
  while(!Bluefruit.connected()) {
    Serial.print(".");
    delay(100);
  }
  Serial.println();
  Serial.println("Connected to Central ................");

  // while connected Central 
  while(Bluefruit.connected())
  {    
    digitalWrite(LED_GREEN, LOW);

    // example data setting
    ud.dataBuff32[0] = millis();
    ud.dataBuff32[1] = micros(); 

    // for Serial plotter 5~20mS
    Serial.print(ud.dataBuff32[0]); Serial.print(" ");
    Serial.println(ud.dataBuff32[1]);

    // write to Characteristic as byte data
    digitalWrite(LED_RED, LOW);        
    myCharacteristic.notify(ud.dataBuff8, dataNum);
    digitalWrite(LED_RED, HIGH);
       
    digitalWrite(LED_GREEN, HIGH);
    delay(1000);      
  } // while connected
      
  // if disconnected from Central
  digitalWrite(LED_GREEN, HIGH);                // connect indicator OFF
  Serial.print("Disconnected from central: ");
} // loop()

//**************************************************************************************************************
// callback function
//*******************************************************************************************************
void connect_callback(uint16_t conn_handle)
{
  Serial.print("【connect_callback】 conn_Handle : ");
  Serial.println(conn_handle, HEX);

  // Get the reference to current connection
  BLEConnection* connection = Bluefruit.Connection(conn_handle);
/*
  Serial.println();
  // request PHY changed to 2MB (2Mbit/sec moduration) 1 --> 2
  Serial.print("Request to change PHY : "); Serial.print(connection->getPHY());
  connection->requestPHY();
  delayMicroseconds(1000000);  // delay a bit for all the request to complete
  Serial.print(" --> "); Serial.println(connection->getPHY());

  // request to update data length  27 --> 251
  Serial.print("Request to change Data Length : "); Serial.print(connection->getDataLength());
  connection->requestDataLengthUpdate();
  delayMicroseconds(1000000);  // delay a bit for all the request to complete
  Serial.print(" --> "); Serial.println(connection->getDataLength());
    
  // request mtu exchange 23 --> 247
  Serial.print("Request to change MTU : "); Serial.print(connection->getMtu());
  connection->requestMtuExchange(dataNum + 3);  // max 247
//  connection->requestMtuExchange(123);  // max 247
  delayMicroseconds(1000000);  // delay a bit for all the request to complete
  Serial.print(" --> "); Serial.println(connection->getMtu());

  // request connection interval  16(20mS) --> 16(20mS)
  Serial.print("Request to change Interval : "); Serial.print(connection->getConnectionInterval());
//  connection->requestConnectionParameter(16); // 20mS(in unit of 1.25) default 20mS
  delayMicroseconds(1000000);  // delay a bit for all the request to complete
  Serial.print(" --> "); Serial.println(connection->getConnectionInterval());
*/
  char central_name[32] = { 0 };
  connection->getPeerName(central_name, sizeof(central_name));

  Serial.print("【connect_callback】 Connected to ");
  Serial.println(central_name);
}

//***********************************************************************************************
void disconnect_callback(uint16_t conn_handle, uint8_t reason)
{
  (void) conn_handle;
  (void) reason;

  Serial.print("【disconnect_callback】 reason = 0x");
  Serial.println(reason, HEX);
}

